<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerPricingPlugin\Entity\Customer;

use Dedi\SyliusCustomerPricingPlugin\Entity\AddressInterface;
use Dedi\SyliusCustomerPricingPlugin\Entity\CustomerInterface;
use Dedi\SyliusCustomerPricingPlugin\Entity\OrderInterface;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Sylius\Component\Resource\Model\TimestampableTrait;

class Organization implements OrganizationInterface
{
    use TimestampableTrait;

    protected ?int $id = null;

    protected string $name;

    protected string $email;

    protected ?AddressInterface $defaultAddress;

    protected Collection $addresses;

    protected Collection $customers;

    protected Collection $orders;

    public function __construct()
    {
        $this->addresses = new ArrayCollection();
        $this->customers = new ArrayCollection();
        $this->orders = new ArrayCollection();
    }

    public function __toString(): string
    {
        return $this->getName();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getName(): string
    {
        return $this->name;
    }

    public function setName(string $name): self
    {
        $this->name = $name;

        return $this;
    }

    public function getEmail(): string
    {
        return $this->email;
    }

    public function setEmail(string $email): self
    {
        $this->email = $email;

        return $this;
    }

    public function getDefaultAddress(): ?AddressInterface
    {
        return $this->defaultAddress;
    }

    public function setDefaultAddress(?AddressInterface $defaultAddress): self
    {
        $this->defaultAddress = $defaultAddress;

        return $this;
    }

    public function getAddresses(): Collection
    {
        return $this->addresses;
    }

    public function addAddress(AddressInterface $address): self
    {
        if (!$this->hasAddress($address)) {
            $this->addresses->add($address);
            $address->setOrganization($this);
        }

        return $this;
    }

    public function removeAddress(AddressInterface $address): self
    {
        $this->addresses->removeElement($address);
        $address->setOrganization();

        return $this;
    }

    public function hasAddress(AddressInterface $address): bool
    {
        return $this->addresses->contains($address);
    }

    public function getCustomers(): Collection
    {
        return $this->customers;
    }

    public function addCustomer(CustomerInterface $customer): self
    {
        if (!$this->hasCustomer($customer)) {
            $this->customers->add($customer);
            $customer->setOrganization($this);
        }

        return $this;
    }

    public function removeCustomer(CustomerInterface $customer): self
    {
        $this->customers->removeElement($customer);
        $customer->setOrganization();

        return $this;
    }

    public function hasCustomer(CustomerInterface $customer): bool
    {
        return $this->customers->contains($customer);
    }

    public function getOrders(): Collection
    {
        return $this->orders;
    }

    public function addOrder(OrderInterface $order): self
    {
        if (!$this->hasOrder($order)) {
            $this->orders->add($order);
            $order->setOrganization($this);
        }

        return $this;
    }

    public function removeOrder(OrderInterface $order): self
    {
        $this->orders->removeElement($order);
        $order->setOrganization();

        return $this;
    }

    public function hasOrder(OrderInterface $order): bool
    {
        return $this->orders->contains($order);
    }
}
