<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerPricingPlugin\EventListener\ORM;

use Dedi\SyliusCustomerPricingPlugin\Entity\ChannelPricing\ChannelPricingSubjectInterface;
use Dedi\SyliusCustomerPricingPlugin\Loader\ChannelPricingLoaderInterface;
use Doctrine\Common\EventSubscriber;
use Doctrine\ORM\EntityManager;
use Doctrine\ORM\Event\PostLoadEventArgs;
use Doctrine\ORM\Events;
use Doctrine\ORM\PersistentCollection;
use Sylius\Component\Core\Model\ChannelPricingInterface;

class ChannelPricingAssociationMetadataListener implements EventSubscriber
{
    public function __construct(
        private readonly EntityManager $entityManager,
        private readonly ChannelPricingLoaderInterface $channelPricingLoader,
    ) {
    }

    public function getSubscribedEvents()
    {
        return [
            Events::postLoad,
        ];
    }

    public function postLoad(PostLoadEventArgs $args): void
    {
        $object = $args->getObject();

        if (!$object instanceof ChannelPricingSubjectInterface) {
            return;
        }

        $this->loadChannelPricingCollection($object);
    }

    private function loadChannelPricingCollection(ChannelPricingSubjectInterface $object): void
    {
        $classMetadata = $this->entityManager->getClassMetadata(get_class($object));

        $associationMappings = $classMetadata->getAssociationMappings();
        $channelPricingAssociationMapping = $associationMappings['channelPricings'];

        $channelPricings = $this->channelPricingLoader->load($object);

        $channelPricingCollection = new PersistentCollection($this->entityManager, $this->entityManager->getClassMetadata(ChannelPricingInterface::class), $channelPricings);
        $channelPricingCollection->setOwner($object, $channelPricingAssociationMapping);
        $object->setChannelPricings($channelPricingCollection);
    }
}
