<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerPricingPlugin\Form\DataTransformer;

use InvalidArgumentException;
use Sylius\Component\Channel\Repository\ChannelRepositoryInterface;
use Sylius\Component\Core\Context\ShopperContextInterface;
use Sylius\Component\Core\Model\ChannelInterface;
use Sylius\Component\Core\Model\ChannelPricingInterface;
use Sylius\Component\Currency\Converter\CurrencyConverterInterface;
use Sylius\Component\Currency\Model\CurrencyInterface;
use Symfony\Component\Form\DataTransformerInterface;

class ChannelPricingConverterTransformer implements DataTransformerInterface
{
    public function __construct(
        private readonly ShopperContextInterface $shopperContext,
        private readonly ChannelRepositoryInterface $channelRepository,
        private readonly CurrencyConverterInterface $currencyConverter,
    ) {
    }

    public function transform(mixed $value): ?ChannelPricingInterface
    {
        if (null === $value) {
            return $value;
        }

        if (!$value instanceof ChannelPricingInterface) {
            throw new InvalidArgumentException('Expected ChannelPricingInterface');
        }

        /** @var ChannelInterface $contextChannel */
        $contextChannel = $this->shopperContext->getChannel();

        /** @var ?ChannelInterface $entityChannel */
        $entityChannel = $this->channelRepository->findOneBy(['code' => $value->getChannelCode()]);

        if (
            null === $entityChannel ||
            $contextChannel === $entityChannel
        ) {
            return $value;
        }

        $contextCurrency = $contextChannel->getBaseCurrency();
        $entityCurrency = $entityChannel->getBaseCurrency();
        if (
            null === $contextCurrency ||
            null === $entityCurrency ||
            $contextCurrency === $entityCurrency
        ) {
            return $value;
        }

        $this->convert($value, $entityCurrency, $contextCurrency);

        return $value;
    }

    public function reverseTransform(mixed $value)
    {
        if (null === $value) {
            return $value;
        }

        if (!$value instanceof ChannelPricingInterface) {
            throw new InvalidArgumentException('Expected ChannelPricingInterface');
        }

        /** @var ChannelInterface $contextChannel */
        $contextChannel = $this->shopperContext->getChannel();

        /** @var ?ChannelInterface $entityChannel */
        $entityChannel = $this->channelRepository->findOneBy(['code' => $value->getChannelCode()]);

        if (
            null === $entityChannel ||
            $contextChannel === $entityChannel
        ) {
            return $value;
        }

        $contextCurrency = $contextChannel->getBaseCurrency();
        $entityCurrency = $entityChannel->getBaseCurrency();
        if (
            null === $contextCurrency ||
            null === $entityCurrency ||
            $contextCurrency === $entityCurrency
        ) {
            return $value;
        }

        $this->convert($value, $contextCurrency, $entityCurrency);

        return $value;
    }

    private function convert(ChannelPricingInterface $value, CurrencyInterface $sourceCurrency, CurrencyInterface $targetCurrency): void
    {
        if (null !== $value->getPrice()) {
            $value->setPrice(
                $this->currencyConverter->convert(
                    $value->getPrice(),
                    $sourceCurrency->getCode(),
                    $targetCurrency->getCode(),
                ),
            );
        }

        if (null !== $value->getOriginalPrice()) {
            $value->setOriginalPrice(
                $this->currencyConverter->convert(
                    $value->getOriginalPrice(),
                    $sourceCurrency->getCode(),
                    $targetCurrency->getCode(),
                ),
            );
        }

        $value->setMinimumPrice(
            $this->currencyConverter->convert(
                $value->getMinimumPrice(),
                $sourceCurrency->getCode(),
                $targetCurrency->getCode(),
            ),
        );
    }
}
