<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerPricingPlugin\Loader\ProductVariant;

use Dedi\SyliusCustomerPricingPlugin\Context\ChannelPricingContextInterface;
use Dedi\SyliusCustomerPricingPlugin\Entity\ChannelPricing\ChannelPricingSubjectInterface;
use Dedi\SyliusCustomerPricingPlugin\Loader\ChannelPricingLoaderInterface;
use Dedi\SyliusCustomerPricingPlugin\Repository\ChannelPricingRepositoryInterface;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Sylius\Component\Core\Context\ShopperContextInterface;
use Sylius\Component\Core\Model\ProductVariantInterface;

class CustomerChannelPricingLoader implements ChannelPricingLoaderInterface
{
    public function __construct(
        private ChannelPricingRepositoryInterface $channelPricingRepository,
        private ChannelPricingContextInterface $channelPricingContext,
        private ShopperContextInterface $shopperContext,
    ) {
    }

    public function load(ChannelPricingSubjectInterface $object): Collection
    {
        $customer = $this->shopperContext->getCustomer();

        if (null === $customer) {
            return new ArrayCollection();
        }

        return $this->channelPricingRepository->findAllForCustomerForProductVariant($object, $customer);
    }

    public function supports(ChannelPricingSubjectInterface $object): bool
    {
        if ($this->channelPricingContext->isAdmin()) {
            return false;
        }

        $customer = $this->shopperContext->getCustomer();

        return $object instanceof ProductVariantInterface && null !== $customer;
    }
}
