<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerPricingPlugin\Validator;

use Dedi\SyliusCustomerPricingPlugin\Entity\ChannelPricing\CustomerGroup\CustomerGroupChannelPricingInterface;
use Dedi\SyliusCustomerPricingPlugin\Repository\ChannelPricingRepositoryInterface;
use Dedi\SyliusCustomerPricingPlugin\Validator\Constraints\UniqueCustomerGroupChannelPricing;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;
use Symfony\Component\Validator\Exception\UnexpectedValueException;

class UniqueCustomerGroupChannelPricingValidator extends ConstraintValidator
{
    public function __construct(
        private readonly ChannelPricingRepositoryInterface $channelPricingRepository,
    ) {
    }

    public function validate(mixed $value, Constraint $constraint)
    {
        if (!$value instanceof CustomerGroupChannelPricingInterface) {
            throw new UnexpectedValueException($value, CustomerGroupChannelPricingInterface::class);
        }

        if (!$constraint instanceof UniqueCustomerGroupChannelPricing) {
            throw new UnexpectedValueException($constraint, UniqueCustomerGroupChannelPricing::class);
        }

        if (null === $value->getProductVariant() ||
            null === $value->getCustomerGroup()
        ) {
            return;
        }

        $existing = $this->channelPricingRepository->findAllForCustomerGroupForProductVariantAndChannel($value->getChannelCode(), $value->getProductVariant(), $value->getCustomerGroup());

        $existing = array_filter($existing, static fn (CustomerGroupChannelPricingInterface $item) => $item->getId() !== $value->getId());

        if (count($existing) > 0) {
            $this->context->buildViolation($constraint->customerGroupChannelPricingAlreadyExistsMessage)
                ->addViolation();
        }
    }
}
